#
#  engine
#
#  Playout Daemon for autoradio project
#
#
#  Copyright (C) 2017-2018 Gottfried Gaisbauer <gottfried.gaisbauer@servus.at>
#
#  This file is part of engine.
#
#  engine is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  any later version.
#
#  engine is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with engine. If not, see <http://www.gnu.org/licenses/>.
#

import os
import sys
import logging

from configparser import ConfigParser


class AuraConfig:
    ini_path = ""
    logger = None

    def __init__(self, ini_path): # = "/etc/aura/engine.ini"):
        self.ini_path = ini_path
        self.logger = logging.getLogger("AuraEngine")
        self.load_config()

    def set(self, key, value):
        """
        Set a property
        @type    key: string
        @param   key: The Key
        @type    value: mixed
        @param   value: Beliebiger Wert
        """
        try:
            self.__dict__[key] = int(value)
        except:
            self.__dict__[key] = str(value)

    # ------------------------------------------------------------------------------------------ #
    def get(self, key, default=None):
        """
        get a loaded property
        @type    key: string
        @param   key: Der Key
        @type    default: mixed
        @param   default: Beliebiger Wert
        """

        if key not in self.__dict__:
            if default:
                self.set(key, default)
            else:
                self.logger.warning("Key " + key + " not found in configfile " + self.ini_path + "!")
                return None

        if key == "loglevel":
            loglvl = self.__dict__[key]

            if loglvl == "debug":
                return logging.DEBUG
            elif loglvl == "info":
                return logging.INFO
            elif loglvl == "warning":
                return logging.WARNING
            elif loglvl == "error":
                return logging.ERROR
            else:
                return logging.CRITICAL

        if key == "debug":
            return self.__dict__[key].count("y")

        return self.__dict__[key]

    # ------------------------------------------------------------------------------------------ #
    def load_config(self):
        """
        Set config defaults and load settings from file
        :return:
        """
        if not os.path.isfile(self.ini_path):
            self.logger.critical(self.ini_path + " not found  :(")
            sys.exit(1)

        # INI einlesen
        f = open(self.ini_path, 'r')
        ini_str = f.read()
        f.close()

        config_parser = ConfigParser()
        try:
            config_parser.read_string(ini_str)
        except Exception as e:
            self.logger.critical("Cannot read " + self.ini_path + "! Reason: " + str(e))
            sys.exit(0)

        for section in config_parser.sections():
            for key, value in config_parser.items(section):
                v = config_parser.get(section, key).replace('"', '').strip()
                self.set(key, v)

        self.set("install_dir", os.path.realpath(__file__ + "../../../.."))
        self.set("use_test_data", False)